package com.example.rotatingdoughnut;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.RectF;
import android.os.Handler;
import android.util.AttributeSet;
import android.view.View;

@SuppressLint("DrawAllocation") 
public class RotatingDoughnut extends View {

	private boolean isMeasured=false;//是否第一次onMeasure
	private int width;//控件宽度
	private int height;//控件高度
	private int doughnutRadius;//外圈大圆环半径
	private int doughnutWidth=7;//圆弧的宽度
	private int pieRadius;//内圈的圆的半径
	private int doughnutSubPie=3;//内圈圆到圆弧的距离
	private float baseScanAngle=60.0f;//圆弧扫过的最小角度，视为基础角度
	private float extraScanAngle=0.0f;//圆弧扫过的角度的增幅，依据正弦曲线变化
	private Paint paint;
	private int doughnutColor=getResources().getColor(R.color.doughnutcolor);//整体颜色
	private int pieColor=getResources().getColor(R.color.doughnutcolor);
	private int SPEED=3;//代表每3毫秒刷新一次
	private float progress=0.0f;//圆弧转过的累计角度
	private float process=2.0f;//圆弧转过角度的基础增幅，实际值根据正弦曲线变化
	
	public RotatingDoughnut(Context context, AttributeSet attrs,
			int defStyleAttr) {
		super(context, attrs, defStyleAttr);
		// TODO Auto-generated constructor stub
	}
	public RotatingDoughnut(Context context, AttributeSet attrs) {
		super(context, attrs);
		// TODO Auto-generated constructor stub
	}
	public RotatingDoughnut(Context context) {
		super(context);
		// TODO Auto-generated constructor stub
	}
	Handler handler=new Handler();
	Runnable run=new Runnable(){

		@Override
		public void run() {
			// TODO Auto-generated method stub
			//wait SPEED毫秒
			handler.postDelayed(run, SPEED);
			//获得每次刷新之后圆弧的起始角度
			progress+=(Math.sin((progress-90f)/180f*Math.PI)*process*1+process*1)*process+process;
			//获得圆弧扫过的角度的增幅
			extraScanAngle=(float) (Math.sin((progress-90f)/180f*Math.PI)*baseScanAngle/2+baseScanAngle/2);
			setProgress(progress);
		}};
	
	@SuppressLint("DrawAllocation") @Override
	protected void onDraw(Canvas canvas) {
		// TODO Auto-generated method stub
		super.onDraw(canvas);
		
		paint=new Paint();
		paint.setColor(doughnutColor);
		paint.setStyle(Style.STROKE);
		paint.setStrokeWidth(doughnutWidth);
		paint.setAntiAlias(true);
		//画出圆弧
		canvas.drawArc(new RectF(doughnutWidth/2,(height-width)/2+doughnutWidth/2,width-doughnutWidth/2,(height-width)/2+width-doughnutWidth/2), -120+progress, 60+extraScanAngle, false, paint);
		
		paint.setStrokeWidth(1);
		//画出最外圈的大圆
		canvas.drawCircle(width/2, height/2, doughnutRadius-1, paint);
		
		pieRadius=doughnutRadius-doughnutWidth/2-doughnutWidth-doughnutSubPie;
		paint.setColor(doughnutColor);
		paint.setStyle(Style.FILL);
		//画出内圈的圆形
		canvas.drawCircle(width/2, height/2, pieRadius, paint);
	}
	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
		// TODO Auto-generated method stub
		super.onMeasure(widthMeasureSpec, heightMeasureSpec);
		if(!isMeasured){
			//获取控件宽高
			width=getMeasuredWidth();
			height=getMeasuredHeight();
			this.doughnutRadius=(width<height?width:height)/2;
		}
	}
	
	public synchronized void setProgress(float progress){
		this.progress=progress;
		//促使控件刷新
		postInvalidate();
	}
	public void doughnutRotating(){
		run.run();
	}
	public void doughnutStop(){
		try {
			run.wait();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	public int getDoughnutWidth() {
		return doughnutWidth;
	}
	public void setDoughnutWidth(int doughnutWidth) {
		this.doughnutWidth = doughnutWidth;
	}
	public int getDoughnutSubPie() {
		return doughnutSubPie;
	}
	public void setDoughnutSubPie(int doughnutSubPie) {
		this.doughnutSubPie = doughnutSubPie;
	}
	public int getSPEED() {
		return SPEED;
	}
	public void setSPEED(int sPEED) {
		SPEED = sPEED;
	}
	public float getProcess() {
		return process;
	}
	public void setProcess(int process) {
		this.process = process;
	}
}